#!/bin/bash

# *** What's Up What's Down Who's Around ***
# *** Version 3.0 - 31st March 2000c     ***
# *** Written by Ross Walker             ***
# *** Copyright Ross Walker 2000	 ***
# *** All Rights Reserved                ***

#Changes from Version 2.1 to 3.0
#**************************************************************************
#1. - Added titling functionality via TITLE title keywords in the 
#     MACHINE-LIST file so the up down table can be divided into groups.
#     Note, current format is TITLE title
#     NO SPACES are allowed in title so use underscores!!!
#2. - Created a config file called CONFIG.WUWDWA.  Currently the 
#     available settings are:
#	i)   ping frequency, set with the keyword PING_FREQ x.
#	ii)  path to wuwdwa.html set with WUWDWAPATH /xxxx/xxxx/xxxx.
#************************************************************************** 

#Changes from Version 2.0 to 2.1
#**************************************************************************
#1 - Added lock file functionality.  The program now checks for a lock
#    lock file before executing to avoid the possiblity of a race condition
#    where the execution frequency is less than the time taken to run the
#    script.
#**************************************************************************

#Changes from Version 1.0 to 2.0
#**************************************************************************
#1 - Changed table display format so cell height is now only height of
#    letters.  Thus more machines are listed on a single page.
#2 - Removed need for UP / DOWN / ERROR graphics by using cell background
#    colour and text.
#3 - Added RUP support to show uptime and load average of machines running
#    the RPC.rstatd daemon.  Consequently the format of the machines file
#    has changed - see below. - NOTE: the rup command is now a required
#    component of WUWDWA-2.
#4 - Reduced ping count to 2 to reduce execution time - this should not
#    prove too big a problem unless their is excessive network traffic.
#**************************************************************************

#This script attempts to ping each machine listed in MACHINE-LIST
#It then writes a web page called wuwdwa.html that lists what machines
#responded and what didn't.  The rup command is also issued on all machines
#that support it as defined in MACHINE-LIST.

#NOTE - The file format for MACHINE-LIST is:		MACHINENAME	IPADDRESS      RUP(=YES or =NO) no spaces in either
#e.g   Testmachine	192.168.0.1	RUP=YES

#THE LINE BELOW MUST BE MODIFIED TO POINT TO THE WEB ACCESS DIRECTORY FOR WUWDWA.html
#WUWDWAPATH="/home/httpd/html/wuwdwa"

#We first check to see if a lock file exists and terminate if it does.
#This stops us getting what is in effect a race condition should the cron
#daemon start a new job before the current has finished.

if [ -f lock ]; then
    #A lock file exists
	echo "Another copy of WUWDWA would appear to be currently running." >&2
	echo "If you are certain that another copy is not currently executing then you should delete the file \"lock\" in the WUWDWA directory and re-execute this script." >&2
	exit 1
fi

#If we have got this far then there is no lock file present so time to create one.
echo " " >> lock

#******************************
#READ SETTINGS FROM CONFIG FILE
#note, the setting WUWDWAPATH
#must point to where the
#wuwdwa.html file is to go.
#******************************

count=0
for CONFIG_SETTING in $( cat CONFIG.WUWDWA ); do
 	let count=count+1
	if [ "$count" = "1" ]; then
		#CONFIG_SETTING contains the variable to set
		CONFIG_VAR_TO_SET=$CONFIG_SETTING
	fi
	if [ "$count" = "2" ]; then
		#CONFIG_SETTING contains the value for the variable
		#Test each allowed setting in turn
		if [ "$CONFIG_VAR_TO_SET" = "PING_FREQ" ]; then
			#We set the ping frequency
			PING_FREQ=$CONFIG_SETTING
		fi
		if [ "$CONFIG_VAR_TO_SET" = "WUWDWAPATH" ]; then
			WUWDWAPATH=$CONFIG_SETTING
		fi
		count=0
	fi	
done

rm -f wuwdwa.html

#We append the file wuwdwa_top.txt to our wuwdwa.html file
cat wuwdwa_top.txt >> wuwdwa.html

#Now we write the time and date to the wuwdwa.html temp file
echo -n $( date ) >> wuwdwa.html
echo "</i></p>" >> wuwdwa.html
echo "<div align=\"center\">" >> wuwdwa.html
echo "<center><table border=\"1\" cellpadding=\"0\" cellspacing=\"0\" width=\"730\">" >> wuwdwa.html
echo "<tr>" >> wuwdwa.html
echo "<td align=\"center\" width=\"140\" valign=\"middle\"><b>Machine Name</b></td>" >> wuwdwa.html
echo "<td align=\"center\" width=\"110\" valign=\"middle\"><b>IP Address</b></td>" >> wuwdwa.html
echo "<td align=\"center\" width=\"58\" valign=\"middle\"><b>Up</b></td>" >> wuwdwa.html
echo "<td align=\"center\" width=\"58\" valign=\"middle\"><b>Down</b></td>" >> wuwdwa.html
echo "<td align=\"center\" width=\"58\" valign=\"middle\"><b>Error</b></td>">> wuwdwa.html
echo "<td align=\"center\" width=\"148\" valign=\"middle\"><b>Uptime</b></td>">> wuwdwa.html
echo "<td align=\"center\" width=\"148\" valign=\"middle\"><b>Load Average</b></td>">> wuwdwa.html
echo "</tr>" >> wuwdwa.html

#Now we loop through each machine in the MACHINE-LIST writing it to the web page

count=0
TITLE_MARKER=false
for MACHPING in $( cat MACHINE-LIST ); do
   let count=count+1
   if [ "$count" = "1" ]; then
        #MACHPING currently contains the machine name or the TITLE keyword - not the ip address
	#Check to see if we have a TITLE keyword
	if [ "$MACHPING" = "TITLE" ]; then
		#We set the title marker to true
		TITLE_MARKER=true
	else
		echo "<tr>" >> wuwdwa.html
		echo "<td align=\"center\" width=\"140\" valign=\"middle\">$MACHPING</td>" >> wuwdwa.html		
	fi
   fi
   if [ "$count" = "2" ]; then
		#WE NEED TO CHECK IF WE ARE LOOKING FOR A TITLE
		if [ "$TITLE_MARKER" = "true" ]; then
			#We write out the titling
			echo "</tr>" >> wuwdwa.html
			echo "<tr>" >> wuwdwa.html
			echo "<td align="left" width="720" valign="middle" colspan="7" bgcolor="#00FF00"><i><b>" >> wuwdwa.html
			echo $MACHPING >> wuwdwa.html
			echo "</b></i></td>" >> wuwdwa.html
			#Reset TITLE_MARKER and counter
			TITLE_MARKER=false
			let count=0
		else
			#EXIT CODES FROM PING 0 = good, 1 = Down, 2 = Error
			#MACHPING now contains the IP Address
			echo "<td align=\"center\" width=\"110\" valign=\"middle\">$MACHPING</td>" >> wuwdwa.html
			ping -c $PING_FREQ $MACHPING > /dev/null
			PINGRESULT=$?
			IPADDRESS=$MACHPING
			if [ $PINGRESULT = 0 ]; then
				#PING WAS SUCCESSFUL
				echo "<td width=\"58\" align=\"center\" valign=\"middle\" bgcolor=\"#00FF00\"><b><font size=\"2\">UP</font></b></td>" >> wuwdwa.html
				echo "<td width=\"58\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">&nbsp;</td>" >> wuwdwa.html
				echo "<td width=\"58\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">&nbsp;</td>" >> wuwdwa.html
			else 	if [ $PINGRESULT = 1 ];  then
					#PING FAILED SYSTEM DOWN
					echo "<td width=\"58\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">&nbsp;</td>" >> wuwdwa.html
					echo "<td width=\"58\" align=\"center\" valign=\"middle\" bgcolor=\"#FF0000\"><b><font size=\"2\">DOWN</font></b></td>" >> wuwdwa.html
					echo "<td width=\"58\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">&nbsp</td>" >> wuwdwa.html
				else
					#PING FAILED ERROR (INVALID IP?)
					echo "<td width=\"58\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">&nbsp;</td>" >> wuwdwa.html
					echo "<td width=\"58\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">&nbsp;</td>" >> wuwdwa.html
					echo "<td width=\"58\" align=\"center\" valign=\"middle\" bgcolor=\"#0000FF\"><b><font size=\"2\">ERROR</font></b></td>" >> wuwdwa.html
				fi
			fi
		fi
   else 
	if [ "$count" = "3" ]; then
		#MACHPING now contains whether or not RUP is supported
		#We only consider RUP if the ping was successful
		if [ $PINGRESULT = "0" ]; then
			#Ping was successful so check if RUP can be used
			#If rup is available then MACHPING should contain RUP=YES
			if [ "$MACHPING" = "RUP=YES" ]; then
				#Now we run the rup command and write what we get back to the variable RUPRESULT
				RUPRESULT=$(rup $IPADDRESS 2>&1)
				if [ "$(echo $RUPRESULT | awk '{print $1}')" = "rup:" ]; then
					#The rup command returned an error
					echo "<td width=\"148\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">Query Failed</td>" >> wuwdwa.html
					echo "<td width=\"148\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">Query Failed</td>" >> wuwdwa.html
				else
					#The variable RUPRESULT must now be queried to obtain the uptime and load averages
					#I am assuming all flavours of Unix return the same rup result format
					#We step through RUPRESULT one column at a time
					echo "<td width=\"148\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">"  >> wuwdwa.html
					let rupcount=0
					rupread="YES"
					for RUPTEST in $(echo "$RUPRESULT"); do
						let rupcount=rupcount+1
						if [ $rupcount -gt 2 ]; then
							#We are now to the point where we write the uptime until we get to load:
							if [ "$RUPTEST" = "load" ]; then
							   rupread="NO"
							fi
							if [ "$rupread" = "YES" ]; then
							   echo "$RUPTEST " >> wuwdwa.html 
							fi
						fi
					done
					echo "</td>" >> wuwdwa.html
					#Now we do the load average
					echo "<td width=\"148\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">" >> wuwdwa.html
					rupread="NO"
					for RUPTEST in $(echo "$RUPRESULT"); do
						if [ "$rupread" = "YES" ]; then
							echo "$RUPTEST " >> wuwdwa.html
						fi
						if [ "$RUPTEST" = "average:" ]; then
							rupread="YES"
						fi
					done
					echo "</td>" >> wuwdwa.html
				fi
			fi
			if [ "$MACHPING" = "RUP=NO" ]; then
				echo "<td width=\"148\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">Not Supported</td>" >> wuwdwa.html
				echo "<td width=\"148\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">Not Supported</td>" >> wuwdwa.html
			else
				if [ "$MACHPING" != "RUP=YES" ]; then
					#THE MACHINE-LIST file is invalid
					echo "EXIT - THE MACHINE-LIST FILE CONTAINS AN INVALID ENTRY"
					echo "OFFENDING STRING = $MACHPING"
					exit 1
				fi
			fi
		else
			#Ping failed so we just write N/A to Uptime and Load Average boxes.
			echo "<td width=\"148\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">N/A</td>" >> wuwdwa.html
			echo "<td width=\"148\" align=\"center\" valign=\"middle\" bgcolor=\"#FFFFFF\">N/A</td>" >> wuwdwa.html
		fi
   	
		echo "</tr>" >> wuwdwa.html
		#Reset the counter to zero
		let count=0
	fi
 fi
done

#Now we put the bottom of the file on
cat wuwdwa_bottom.txt >> wuwdwa.html

#Now we copy the file over the live web one - NOTE THIS LINE NEEDS TO BE
#CHANGED TO MATCH YOUR SYSTEM SETUP.

cp -f wuwdwa.html $WUWDWAPATH/index.html

#Last thing we do is remove the lock file
rm -f lock
exit 0

